<?php

namespace Drupal\vide_bg\Form;

use Drupal\block\Entity\Block;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Class VideConfigForm.
 */
class VideConfigForm extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [
      'vide_bg.settings',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'vide_config_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('vide_bg.settings');
    $form['fs_bg_files'] = array(
      '#type' => 'details',
      '#title' => 'Background video/image files',
    );
    $form['fs_bg_files']['vide_bg_file_mp4'] = array(
      '#title' => $this->t('mp4'),
      '#type' => 'managed_file',
      '#default_value' => [$config->get('vide_bg_file_mp4')],
      '#upload_location' => 'public://vide_bg/',
      '#upload_validators' => array(
        'file_validate_extensions' => array('mp4'),
        'file_validate_size' => array(50 * 1024 * 1024),
      ),
    );
    $form['fs_bg_files']['vide_bg_file_webm'] = array(
      '#title' => $this->t('webm'),
      '#type' => 'managed_file',
      '#default_value' => [$config->get('vide_bg_file_webm')],
      '#upload_location' => 'public://vide_bg/',
      '#upload_validators' => array(
        'file_validate_extensions' => array('webm'),
        'file_validate_size' => array(50 * 1024 * 1024),
      ),
    );
    $form['fs_bg_files']['vide_bg_file_ogg'] = array(
      '#title' => $this->t('ogg'),
      '#type' => 'managed_file',
      '#default_value' => [$config->get('vide_bg_file_ogg')],
      '#upload_location' => 'public://vide_bg/',
      '#upload_validators' => array(
        'file_validate_extensions' => array('ogv'),
        'file_validate_size' => array(50 * 1024 * 1024),
      ),
    );
    $form['fs_bg_files']['vide_bg_file_poster'] = array(
      '#title' => $this->t('poster:'),
      '#type' => 'managed_file',
      '#upload_location' => 'public://vide_bg/',
      '#default_value' => [$config->get('vide_bg_file_poster')],
      '#upload_validators' => array(
        'file_validate_extensions' => array('jpg jpeg gif png svg bmp tiff pcx webp'),
        'file_validate_size' => array(50 * 1024 * 1024),
      ),
    );

    $form['fs_vide_bg_options'] = array(
      '#type' => 'details',
      '#title' => 'Options',
    );
    $vide_type_bg_position = array(
      0 => 'Disabled',
      1 => 'Body',
      2 => 'Block',
      3 => 'Html element',
    );
    $form['fs_vide_bg_options']['vide_bg_type_bg_position'] = array(
      '#title' => $this->t('Background:'),
      '#type' => 'radios',
      '#options' => $vide_type_bg_position,
      '#default_value' => $config->get('vide_bg_type_bg_position'),
    );
    $form['fs_vide_bg_options']['vide_bg_bg_block_id'] = array(
      '#type' => 'select',
      '#title' => $this->t('Select block for background'),
      '#default_value' => $config->get('vide_bg_bg_block_id'),
      '#options' => $this->vide_bg_load_all_blocks(),
      '#states' => array(
        'visible' => array(
          ':input[name="vide_bg_type_bg_position"]' => array('value' => 2),
        ),
      ),
    );
    $form['fs_vide_bg_options']['vide_bg_bg_html_element'] = array(
      '#type' => 'textfield',
      '#title' => $this->t('Html identifier'),
      '#default_value' => $config->get('vide_bg_bg_html_element'),
      '#description' => t('Enter the id or class name of html element (for ex. #idname or .classname)'),
      '#states' => array(
        'visible' => array(
          ':input[name="vide_bg_type_bg_position"]' => array('value' => 3),
        ),
      ),
    );
    $form['fs_vide_bg_options']['vide_bg_volume'] = array(
      '#type' => 'textfield',
      '#title' => $this->t('Volume'),
      '#default_value' => $config->get('vide_bg_volume'),
    );
    $form['fs_vide_bg_options']['vide_bg_playback_rate'] = array(
      '#type' => 'textfield',
      '#title' => $this->t('Playback Rate'),
      '#default_value' => $config->get('vide_bg_playback_rate'),
    );

    $active = array(0 => 'No', 1 => 'Yes');
    $form['fs_vide_bg_options']['vide_bg_muted'] = array(
      '#type' => 'radios',
      '#options' => $active,
      '#title' => $this->t('Muted'),
      '#default_value' => $config->get('vide_bg_muted'),
    );
    $form['fs_vide_bg_options']['vide_bg_loop'] = array(
      '#type' => 'radios',
      '#options' => $active,
      '#title' => $this->t('Loop'),
      '#default_value' => $config->get('vide_bg_loop'),
    );
    $form['fs_vide_bg_options']['vide_bg_autoplay'] = array(
      '#type' => 'radios',
      '#options' => $active,
      '#title' => $this->t('Autoplay'),
      '#default_value' => $config->get('vide_bg_autoplay'),
    );
    $form['fs_vide_bg_options']['vide_bg_position'] = array(
      '#type' => 'textfield',
      '#title' => $this->t('Position'),
      '#default_value' => $config->get('vide_bg_position'),
      '#description' => $this->t('Similar to the CSS `background-position` property'),
    );
    $form['fs_vide_bg_options']['vide_bg_resizing'] = array(
      '#type' => 'radios',
      '#options' => $active,
      '#title' => $this->t('Resizing'),
      '#default_value' => $config->get('vide_bg_resizing'),
    );
    $form['fs_vide_bg_options']['vide_bg_bg_color'] = array(
      '#type' => 'textfield',
      '#title' => $this->t('Background color'),
      '#default_value' => $config->get('vide_bg_bg_color'),
      '#description' => $this->t('Allow custom background-color for Vide div'),
    );
    $form['fs_vide_bg_options']['vide_bg_class_name'] = array(
      '#type' => 'textfield',
      '#title' => $this->t('Class name'),
      '#default_value' => $config->get('vide_bg_class_name'),
      '#description' => $this->t('Add custom CSS class to Vide div'),
    );

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    parent::validateForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    parent::submitForm($form, $form_state);
    $config = $this->config('vide_bg.settings');
    $file_ids = array(
      'vide_bg_file_mp4',
      'vide_bg_file_webm',
      'vide_bg_file_ogg',
      'vide_bg_file_poster',
    );
    foreach ($file_ids as $file_id) {
      $image = $form_state->getValue($file_id);
      $file = \Drupal\file\Entity\File::load($image[0]);
      if(!empty($file)) {
        $file->setPermanent();
        $file->save();
        $file_usage = \Drupal::service('file.usage');
        $file_usage->add($file, 'vide_bg', 'user', \Drupal::currentUser()->id());
        $config->set($file_id, $form_state->getValue($file_id)[0]);
        // Get extension of the vide_file_poster file.
        if ($file_id === 'vide_bg_file_poster') {
          $poster_file_extension = $this->vide_bg_poster_file_extension($image[0]);
          $config->set('vide_bg_poster_type', $poster_file_extension);
        }
      }
    }

    if ($form_state->getValue('vide_bg_type_bg_position') === "2") {
      $vide_bg_block_delta_array = explode('--', $form_state->getValue('vide_bg_bg_block_id'));
      $config->set('vide_bg_bg_block_delta', $vide_bg_block_delta_array[1]);
    }

    $config->set('vide_bg_type_bg_position', $form_state->getValue('vide_bg_type_bg_position'));
    $config->set('vide_bg_bg_block_id', $form_state->getValue('vide_bg_bg_block_id'));
    $config->set('vide_bg_bg_html_element', $form_state->getValue('vide_bg_bg_html_element'));
    $config->set('vide_bg_volume', $form_state->getValue('vide_bg_volume'));
    $config->set('vide_bg_playback_rate', $form_state->getValue('vide_bg_playback_rate'));
    $config->set('vide_bg_muted', $form_state->getValue('vide_bg_muted'));
    $config->set('vide_bg_loop', $form_state->getValue('vide_bg_loop'));
    $config->set('vide_bg_autoplay', $form_state->getValue('vide_bg_autoplay'));
    $config->set('vide_bg_position', $form_state->getValue('vide_bg_position'));
    $config->set('vide_bg_resizing', $form_state->getValue('vide_bg_resizing'));
    $config->set('vide_bg_bg_color', $form_state->getValue('vide_bg_bg_color'));
    $config->set('vide_bg_class_name', $form_state->getValue('vide_bg_class_name'));
    $config->save();
  }

  /**
   * Loads all blocks' information from the database.
   *
   * @return array
   *   An array of blocks grouped by region.
   */
  public function vide_bg_load_all_blocks() {
    $blocks = \Drupal\block\Entity\Block::loadMultiple();
    foreach ($blocks as $block) {
      $block_id = $block->id();
      $block_label = $block->label();
      $blocks_list["block--{$block_id}"] = $block_label;
    }
    return $blocks_list;
  }

  /**
   * Get the file extension by fid.
   */
  public function vide_bg_poster_file_extension($fid) {
    if (!$fid) {
      return 'none';
    }
    $extension = 'detect';
    $file = \Drupal\file\Entity\File::load($fid);
    if (preg_match('/.+\.([A-Za-z]+)/', $file->label(), $output)) {
      $extension = mb_strtolower($output[1]);
    }
    return $extension;
  }

}
